/*
 * Copyright (c) 2018 Samsung Electronics Co., Ltd.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include <dlog.h>

#include <string>
#include <memory>
#include <map>

#include "screen_connector_remote_surface/window_context.h"
#include "screen_connector_remote_surface/common.h"

#ifdef LOG_TAG
#undef LOG_TAG
#endif

#define LOG_TAG "SC_WINDOW_CONTEXT"

namespace screen_connector {

WindowContext::WindowContext() :
  ecore_event_handler_(new EcoreEventHandler(this)) {
}

void WindowContext::UpdateWindow(unsigned int win, bool fully_obscured) {
  win_map_[win] = fully_obscured;
}

void WindowContext::RemoveWindow(unsigned int win) {
  if (win_map_.find(win) == win_map_.end())
    return;
  win_map_.erase(win);
}

void WindowContext::OnWindowShow(unsigned int win) {
  UpdateWindow(win, false);
  LOGD("[__WINDOW_SHOW__] win(%u)", win);
}

void WindowContext::OnWindowHide(unsigned int win) {
  RemoveWindow(win);
  LOGD("[__WINDOW_HIDE__] win(%u)", win);
}

void WindowContext::OnWindowVisibilityChange(unsigned int win,
                                             bool fully_obscured) {
  UpdateWindow(win, fully_obscured);
  LOGD("[__WINDOW_VISIBILITY_CHANGE__] win(%u), fully_obscured(%d)",
      win, fully_obscured);
}

void WindowContext::OnWindowPreVisibilityChange(unsigned int win,
                                                bool fully_obscured) {
  UpdateWindow(win, fully_obscured);
  LOGD("[__WINDOW_PRE_VISIBILITY_CHANGE__] win(%u), fully_obscured(%d)",
      win, fully_obscured);
}

}  // namespace screen_connector
