/*
 * Copyright (c) 2017 Samsung Electronics Co., Ltd.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include "screen_connector_remote_surface_evas/watcher_image_internal.h"

#define SC_TOOLKIT_HANDLE_TAG "SC_TOOLKIT_HANDLE_TAG"

namespace screen_connector {

WatcherImage::WatcherImage(Evas_Object* raw, const std::string& instId, int pid,
                           void* tag) : EvasObject(raw, true) {
  evas_object_image_alpha_set(GetRaw(), 1);
}

WatcherImage::~WatcherImage() {
}

void WatcherImage::UpdateTbm(std::shared_ptr<WlBuffer> tbm) {
  int width;
  int height;
  tbm_surface_h tbmSurface;
  Evas_Native_Surface ns;

  prev_buf_ = std::move(tbm);
  /* get tbm surface from buffer */
  tbmSurface = (tbm_surface_h)wl_buffer_get_user_data(prev_buf_->GetRaw());
  width = tbm_surface_get_width(tbmSurface);
  height = tbm_surface_get_height(tbmSurface);

  memset(&ns, 0, sizeof(Evas_Native_Surface));
  ns.version = EVAS_NATIVE_SURFACE_VERSION;
  ns.type = EVAS_NATIVE_SURFACE_TBM;
  ns.data.tbm.buffer = tbmSurface;

  evas_object_resize(GetRaw(), width, height);
  evas_object_image_size_set(GetRaw(), width, height);
  evas_object_image_native_surface_set(GetRaw(), &ns);
  evas_object_image_pixels_dirty_set(GetRaw(), EINA_TRUE);
}

void WatcherImage::ClearTbm() {
     /* set null to previous object for the tbm type */
    evas_object_image_native_surface_set(GetRaw(), NULL);
    prev_buf_.reset();
}

void WatcherImage::UpdateFile(int fd, uint32_t size) {
  void* map = mmap(NULL, size, PROT_READ, MAP_SHARED, fd, 0);
  char format[] = "png";
  int width, height;

  evas_object_image_memfile_set(GetRaw(), map, size, format, NULL);
  evas_object_image_size_get(GetRaw(), &width, &height);
  evas_object_image_fill_set(GetRaw(), 0, 0, width, height);
  evas_object_resize(GetRaw(), width, height);
  munmap(map, size);
}

}  // namespace screen_connector
